//=============================================================================
// Name        : IRP2D.cpp
// Author      : Pedro Henrique Del Bianco Hokama
// Version     : v1.0
// Copyright   : Free for research, with proper citation
// Description : This is the code used in the paper from SBPO
//               This code aims to solve the (constrained) inventory-routing
//               problem with two dimensional loading constraints.
//               In a instance of this problem, it is given:
//               n = number_of_clients
//               m = number_of_item_types
//               h = horizon
//               vertex 0 is the depot
//               (x_i, y_i) coordinates of vertex i
//               (W, H) dimensions of the container
//               (w_b, h_b) dimensions of each item of type b
//               di_b^p number of items of type b to client i in period p
//               A_i the max area in the client to stock (geometry not considered)
//               V = number of vehicles needed
//
//               The file format is as follows:
//
//               n m h
//               x_0 y_0
//               W H
//               0 w_0 h_0
//               ...
//               m w_m h_m
//               1 x_1 y_1 d1_0^1..d1_0^h d1_1^1..d1_1^h..d1_m^1..d1_m^h A_1
//               ...
//               n x_n y_n dn_0^1..dn_0^h dn_1^1..dn_1^h..dn_m^1..dn_m^h A_n
//               V
//
//               Distances between vertices are integers truncated.
//               You may contact me by email which is composed by my last
//               name followed by a @ within the domain ic.unicamp.br
//=============================================================================

#include <iostream>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <string>

#include "irp2d_instance.h"
#include "irp2d_solver_exact.h"

using namespace std;
using namespace IRP2D;

void print_usage(){
  cout << "./IRP [REQ_ARGS|OPTIONS] [FILE]" << endl;
  cout << "REQUIRED ARGUMENTS: " << endl;
  cout << "  -i <instance_file>" << endl;
  cout << "OPTIONS: " << endl;
  cout << "  -t Threads  |  default: 4" << endl;
  cout << "  -v Verbose  |  default: 1" << endl;
  cout << "  -m Method  |  default: \"exact\"" << endl;
  cout << "  -l time_Limit in seconds  |  default: -1 (unlimited)" << endl;
  cout << "  -p time_for_packing in seconds  |  default: -1 (unlimited)" << endl;
  cout << "  -o Output_path  |  default: \".\"" << endl;
  cout << "  -u tUnning mode" << endl;
}

int main(int argc, char **argv) {
  //Verbose levels are:
  // 0 - none
  // 1 - minimal
  // 2 - detailed with pdf
  // 3 - debug

  //Setting default values
  int verbose = 1;
  int threads = 4;
  string method("exact");
  string instance_file_name("");
  string output_path("./");
  string instance_type("mmirp2d");
  double time_limit = -1;
  double time_for_packing_cp_solver = -1;
  bool error_flag = false;

  //Parameters reading
  int c;
  opterr = 1;
  while ((c = getopt (argc, argv, "t:v:l:p:m:o:i:")) != -1)
    switch (c){
    case 't':
      threads = atoi(optarg);
      break;
    case 'v':
      verbose = atoi(optarg);
      break;
    case 'l':
      time_limit = atof(optarg);
      break;
    case 'p':
      time_for_packing_cp_solver = atof(optarg);
      break;
    case 'm':
      method = optarg;
      break;
    case 'o':
      output_path = optarg;
      break;
    case 'i':
      instance_file_name = optarg;
      break;
    case '?':
      if (optopt == 't' || optopt == 'v' || optopt == 'l' || optopt == 'p' ||
          optopt == 'm' || optopt == 'o' || optopt == 'i'){
        cerr << "Option -" << optopt << " requires an argument." << endl;
        print_usage();
        return 0;
      } else if (isprint (optopt)){
        cerr << "Unknown option '-" << optopt << "'." << endl;
        print_usage();
        return 0;
      } else {
        cerr << "Unknown option character '" << optopt << "'." << endl;
        print_usage();
        return 1;
      }
    default:
      abort ();
    }

  //parameter validations
  for (int index = optind; index < argc; index++){
    cerr << "Non-option argument " << argv[index] << endl;
    error_flag = true;
  }

  if(instance_file_name.compare("") == 0){
    cerr << "Instance file \"-i <instance_file>\" is required." << endl;
    error_flag = true;
  }

  if(error_flag == true){
    print_usage();
    cerr << "Parameter errors encountered, program will be terminated!" << endl;
    exit(-1);
  }


  Irp2dInstance instance;
  instance.set_output_path(output_path);
  instance.set_verbose(verbose);
  instance.readInstance(instance_file_name);

  if (verbose >= 1) {
    cout << "Instance: " << instance.get_file_base_name() << endl;
    cout << "Method: " << method << endl;
    cout << "Verbose: " << verbose   << endl;
    cout << "Threads: " << threads << endl;
    cout << "Time Limit: " << time_limit << endl;
    cout << "Time Limit for Packing: " << time_for_packing_cp_solver << endl;
  }

  if (verbose >= 2) cout << instance;

  Irp2dSolverExact solver_exact;
  solver_exact.set_threads(threads);
  solver_exact.set_verbose(verbose);
  solver_exact.set_time_limit(time_limit);
  solver_exact.set_time_packing_cp_solver_limit(time_for_packing_cp_solver);
  solver_exact.solve(instance);

  cout.precision(2);
  cout << fixed;
  cout << "Total program time: " << solver_exact.get_total_time() << endl;


  return 0;
}
