/*
 * irp_solver_exact.cpp
 *
 *  Created on: 27 de jul de 2016
 *      Author: hokama
 */

#include "gurobi_c++.h"
#include "ilcplex/ilocplex.h"
#include "ilcp/cp.h"
#include <iomanip>
#include <set>

#include "irp2d_solver_exact.h"
#include "irp2d_subtourelim2d.h"

namespace IRP2D {

//This is just to make the boring terminal a little bit
//colorful
#define Green if (getenv("TERM") != NULL) cout << "\033[1;32m";
#define Green_ if (getenv("TERM") != NULL) cout << "\033[4;32m";
#define DefaultColor if (getenv("TERM") != NULL) cout << "\033[0m";

void generate_pdf(string file, Irp2dInstance *inst, vector < vector < GRBVar * > > x,
    vector < vector < GRBVar * > > y,
    vector < vector < vector < GRBVar * > > > q, int verbose);

//Constructor sets some default arguments
Irp2dSolverExact::Irp2dSolverExact() {
  verbose = 1;
  threads = 4;
  total_time = 0;
  time_limit = -1;
  time_for_packing_cp_solver = -1;
}

Irp2dSolverExact::~Irp2dSolverExact() {
}


int Irp2dSolverExact::solve(Irp2dInstance &insto){
  if(verbose >= 2) cout << "Starting solver" << endl;

  Irp2dInstance* inst = static_cast < Irp2dInstance* > (&insto);

  int number_of_vertices = inst->get_number_of_customers() + 1;
  int number_of_edges = (number_of_vertices * (number_of_vertices - 1))/2;

  GRBEnv* env = 0;

  //Decision variables
  //x[e][k][t] is 1 if edge e is used by vehicle k in period t, 0 otherwise.
  vector < vector < GRBVar * > > x(number_of_edges);
  //y[0][k][t] is 1 if vehicle k is used in period t, 0 otherwise.
  //y[c][k][t] is 1 if vehicle k attends customer c in period t, 0 otherwise.
  vector < vector < GRBVar * > > y(number_of_vertices);
  //q[c][t'][k][t] is 1 if vehicle k attends the demand t' of customer c in period t, 0 otherwise.
  vector < vector < vector < GRBVar * > > > q(number_of_vertices);
  //period is given by t = 1, ..., inst->get_horizon()
  // t = 0 is the initial status of the instance
  //vehicles are given by k = 0, ..., inst->get_number_of_vehicles() - 1
  //edges are given by e = 0, ..., number_of_edges - 1
  //clients are given by v = 1, ..., number_of_vertices - 1
  //where number_of_vertices = inst->get_number_of_customers() + 1
  //vertex v = 0 is the supplier

  try{
    // Model
    env = new GRBEnv();
    GRBModel model = GRBModel(*env);

    model.set(GRB_StringAttr_ModelName, "IRP");
    model.update();

    //must disable dual reductions when using lazy constraints
    model.getEnv().set(GRB_IntParam_LazyConstraints, 1);
    model.getEnv().set(GRB_IntParam_DualReductions, 0);
    if(verbose <= 1) model.getEnv().set(GRB_IntParam_OutputFlag, 0);
    model.getEnv().set(GRB_IntParam_Threads, threads);
    model.getEnv().set(GRB_DoubleParam_NodefileStart, 1); //Memory limit per threads (in GB)
    if (get_time_limit() > 0) model.getEnv().set(GRB_DoubleParam_TimeLimit, get_time_limit());

    // *********************************
    //adding all variables to the model
    if(verbose >= 2) cout << "Adding Variables" << endl;
    for (int edge = 0; edge < number_of_edges; edge++){
      x[edge].resize(inst->get_number_of_vehicles());
      for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        x[edge][vehicle] = model.addVars(inst->get_horizon()+1, GRB_SEMIINT);
        //note that x[edge][vehicle][0] is added but should not be used;
      }
    }
    for (int vertex = 1; vertex < number_of_vertices; vertex++){
      q[vertex].resize(inst->get_horizon()+1);
      for (int period = 1; period <= inst->get_horizon(); period++){
        q[vertex][period].resize(inst->get_number_of_vehicles());
        for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
          q[vertex][period][vehicle] = model.addVars(period+1, GRB_BINARY);
        }
      }
    }

    for (int vertex = 0; vertex < number_of_vertices; vertex++){
      y[vertex].resize(inst->get_number_of_vehicles());
      for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        y[vertex][vehicle] = model.addVars(inst->get_horizon()+1, GRB_BINARY);
        //note that y[vertex][vehicle][0] is added but should not be used;
      }
    }

    model.update();
    // *********************************


    // *********************************
    //removing non used vars
    if(verbose >= 2) cout << "Removing unused variables" << endl;
    for (int edge = 0; edge < number_of_edges; edge++){
      for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        model.remove(x[edge][vehicle][0]);
      }
    }
    for (int vertex = 0; vertex < number_of_vertices; vertex++){
      for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        model.remove(y[vertex][vehicle][0]);
      }
    }
    for (int vertex = 1; vertex < number_of_vertices; vertex++){
      for (int period = 1; period <= inst->get_horizon(); period++){
        for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
          model.remove(q[vertex][period][vehicle][0]);
        }
      }
    }

    model.update();
    // *********************************


    // *********************************
    //setting variables attributes
    //edges from supplier
    if(verbose >= 2) cout << "Configuring Variables" << endl;
    for (int client = 1; client < number_of_vertices; client++){
      //edges leaving depot
      int edge = inst->get_edge_id(0, client);
      for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        for (int period = 1; period <= inst->get_horizon(); period++){
          ostringstream vname;
          vname << "x[" << edge << "][" << vehicle << "][" << period << "]";
          x[edge][vehicle][period].set(GRB_StringAttr_VarName,vname.str());
          x[edge][vehicle][period].set(GRB_DoubleAttr_LB, 0.0);
          x[edge][vehicle][period].set(GRB_DoubleAttr_UB, 2.0); //upper bound 2
          x[edge][vehicle][period].set(GRB_DoubleAttr_Obj, inst->get_distance(0, client));
        }
      }
    }

    //remaining edges
    for (int i = 1; i < number_of_vertices; i++){
      for (int j = i+1; j < number_of_vertices; j++){
        int edge = inst->get_edge_id(i, j);
        for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
          for (int period = 1; period <= inst->get_horizon(); period++){
            ostringstream vname;
            vname << "x[" << edge << "][" << vehicle << "][" << period << "]";
            x[edge][vehicle][period].set(GRB_StringAttr_VarName,vname.str());
            x[edge][vehicle][period].set(GRB_DoubleAttr_LB, 0.0);
            //upper bound 1 if it obeys triangle inequality
            x[edge][vehicle][period].set(GRB_DoubleAttr_UB, 1.0);
            x[edge][vehicle][period].set(GRB_DoubleAttr_Obj, inst->get_distance(i, j));
          }
        }
      }
    }
    //vehicles usage
    for (int vertex = 0; vertex < number_of_vertices; vertex++){
      for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        for (int period = 1; period <= inst->get_horizon(); period++){
          ostringstream vname;
          vname << "y[" << vertex << "][" << vehicle << "][" << period << "]";
          y[vertex][vehicle][period].set(GRB_StringAttr_VarName,vname.str());
          y[vertex][vehicle][period].set(GRB_DoubleAttr_LB, 0.0);
          y[vertex][vehicle][period].set(GRB_DoubleAttr_UB, 1.0);
        }
      }
    }
    //demands attended
    for (int vertex = 1; vertex < number_of_vertices; vertex++){
      for (int period = 1; period <= inst->get_horizon(); period++){
        for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
          for (int t = 1; t <= period; t++){
            ostringstream vname;
            vname << "q[" << vertex << "][" << period << "][" << vehicle << "][" << t << "]";
            q[vertex][period][vehicle][t].set(GRB_StringAttr_VarName,vname.str());
            q[vertex][period][vehicle][t].set(GRB_DoubleAttr_LB, 0.0);
            q[vertex][period][vehicle][t].set(GRB_DoubleAttr_UB, 1.0);
          }
        }
      }
    }
    model.update();
    // *********************************


    // *********************************
    //adding constraints

    //area of vehicle
    for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
      for (int period = 1; period <= inst->get_horizon(); period++){
        GRBLinExpr total_delivered_by_vehicle_in_this_period = 0;
        for (int t_demand = period; t_demand <= inst->get_horizon(); t_demand++){
          for (int prod = 0; prod < inst->get_number_of_products(); prod++){
            for (int customer = 1; customer < number_of_vertices; customer++){
              total_delivered_by_vehicle_in_this_period +=
                  q[customer][t_demand][vehicle][period] *
                  inst->get_demand_of_product_per_customer_per_period(customer, prod, t_demand) *
                  inst->get_width(prod) * inst->get_height(prod);
            }
          }
        }
        ostringstream cname;
        cname << "vehicle_" << vehicle << "_area_capacity_at_period_" << period;
        model.addConstr(total_delivered_by_vehicle_in_this_period <=
            (double) inst->get_vehicle_width() * inst->get_vehicle_height() * y[0][vehicle][period], cname.str());
      }
    }
    //assures that demands will be attended
    for (int customer = 1; customer < number_of_vertices; customer++){
      for (int t_demand = 1; t_demand <= inst->get_horizon(); t_demand++){
        int total_demand = 0;
        for (int prod = 0; prod < inst->get_number_of_products(); prod++){
          total_demand += inst->get_demand_of_product_per_customer_per_period(customer, prod, t_demand);
        }
        if(total_demand > 0){
          GRBLinExpr total_vehicle_used = 0;
          for (int period = 1; period <= t_demand; period++){
            for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
              total_vehicle_used += q[customer][t_demand][vehicle][period];
            }
          }
          ostringstream cname;
          cname << "no_split_" << customer << "_" << t_demand;
          model.addConstr(total_vehicle_used == 1, cname.str()); //no split

        }
      }
    }

    //demands only can be attended if client is visited by vehicle
    for (int customer = 1; customer < number_of_vertices; customer++){
      for (int t_demand = 1; t_demand <= inst->get_horizon(); t_demand++){
        for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
          for (int period = 1; period <= t_demand; period++){
            ostringstream cname;
            cname << "demand_" << customer << "_" << t_demand << "_attend_by_vehicle_" << vehicle << "_in_period_" << period;
            model.addConstr(q[customer][t_demand][vehicle][period] <= y[customer][vehicle][period], cname.str());
          }
        }
      }
    }

    //vehicle should only visit a client to attend some demand
    //this is redundant since minimization should assures that
    /*for (int customer = 1; customer < number_of_vertices; customer++){
      for (int period = 1; period <=  inst->get_horizon(); period++){
        for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
          GRBLinExpr total_demand = 0;
          ostringstream cname;
          cname << "total_demand_" << customer << "_attend_by_vehicle_" << vehicle << "_in_period_" << period;
          for (int t_demand = period; t_demand <= inst->get_horizon(); t_demand++){
            total_demand += q[customer][t_demand][vehicle][period];
          }
          model.addConstr(total_demand >= y[customer][vehicle][period], cname.str());
        }
      }
    }*/


    //total delivered products (orders) cannot exceed max area of customer
    for (int customer = 1; customer < number_of_vertices; customer++){
      for (int period = 1; period <=  inst->get_horizon(); period++){
        GRBLinExpr stock_exp = 0;
        ostringstream cname;
        cname << "stock_of_customer_" << customer << "_in_period_" << period;
        for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
          for (int past = 1; past <= period; past++){
            for (int future = period; future <= inst->get_horizon(); future++){
              int total_area_of_this_order = 0;
              for (int product = 0; product < inst->get_number_of_products(); product++){
                total_area_of_this_order +=
                  inst->get_demand_of_product_per_customer_per_period(customer, product, future)
                * inst->get_width(product) * inst->get_height(product);
              }
              stock_exp += q[customer][future][vehicle][past] * total_area_of_this_order;
            }
          }
        }
        model.addConstr(stock_exp <= inst->get_max_area_customer(customer), cname.str());
      }
    }

    //edges visited customers
    for (int i = 0; i < number_of_vertices; i++){
      for (int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        for (int period = 1; period <= inst->get_horizon(); period++){
          GRBLinExpr edges_used_by_vehicle_in_customer = 0;
          for (int j = 0; j < number_of_vertices; j++){
            if(i != j) edges_used_by_vehicle_in_customer += x[inst->get_edge_id(i, j)][vehicle][period];
          }
          ostringstream cname;
          cname << "degree_per_vertex_per_vehicle_" << i << "_" << vehicle << "_" << period;
          model.addConstr(edges_used_by_vehicle_in_customer == 2 * y[i][vehicle][period], cname.str());
        }
      }
    }

    subtourelim2d ste = subtourelim2d(x, y, q, inst, verbose, time_for_packing_cp_solver);
    model.setCallback(&ste);

    model.update();

    model.write("saida.lp");

    if(verbose >= 1) cout << "Number of variables: " << model.get(GRB_IntAttr_NumVars) << endl;
    if(verbose >= 1) cout << "Number of initial inequalities: " << model.get(GRB_IntAttr_NumConstrs) << endl;


    if(verbose >= 2) cout << "Starting Gurobi" << endl;
    model.optimize();
    total_time = model.get(GRB_DoubleAttr_Runtime);

    if(verbose >= 1) {
      cout.precision(2);
      cout << fixed;
      cout << "Solution: " << model.get(GRB_DoubleAttr_ObjVal) << endl;
      cout << "Gap: " << model.get(GRB_DoubleAttr_MIPGap) * 100 << "%" << endl;
      cout << "Time gurobi solver: " << model.get(GRB_DoubleAttr_Runtime) << endl;
      cout << "Number of explored nodes: " << (int) model.get(GRB_DoubleAttr_NodeCount) << endl;
    }

    /////////////////////////////////////////////////////////////////////////////////////
    //printing solution
    if (verbose >= 2){
      double total_cost = 0;
      cout << endl;

      for(int customer = 1; customer < inst->get_number_of_customers()+1; customer++){
        Green
        cout << "At the customer " << customer << ": " << endl;
        cout << "prod |     level of product in period" << endl;
        Green_
        cout << "     |";
        for (int period = 0; period < inst->get_horizon()+1; period++){
          cout << setw(8) << period;
        }
        cout  << endl;
        DefaultColor
        for(int prod = 0; prod < inst->get_number_of_products(); prod++){
          cout << setw(4) << prod << " |";
          for (int period = 0; period < inst->get_horizon() + 1; period++){
            stringstream ss;
            if (period != 0) {
              ss << "+" << inst->get_demand_of_product_per_customer_per_period(customer, prod, period);;
            }
            ss << "-" << inst->get_demand_of_product_per_customer_per_period(customer, prod, period);
            cout << setw(8) << ss.str();
          }
          cout << endl;
        }
        cout << endl;
      }

      //vehicles used each period
      cout << endl;
      Green_
      cout << "vehicle | ";
      for (int period = 1; period < inst->get_horizon()+1; period++){
        cout << setw(5) << period;
      }
      cout  << endl;
      DefaultColor
      for(int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
        cout << setw(7) << vehicle << "   ";
        for (int period = 1; period < inst->get_horizon() + 1; period++){
          cout << setw(5) << (int) floor(y[0][vehicle][period].get(GRB_DoubleAttr_X)+0.05);
        }
        cout << endl;
      }

      //Total edges used each period
      for (int period = 1; period < inst->get_horizon() + 1; period++){
        cout << endl;
        Green
        cout << "Edges at period: " << period << endl;
        Green_
        cout << setw(5) << "i/j";
        for(int i = 0; i < inst->get_number_of_customers() + 1; i++){
          cout << setw(5) << i;
        }
        cout << endl;
        DefaultColor
        for(int i = 0; i < inst->get_number_of_customers() + 1; i++){
          Green
          cout << setw(3) << i << " |";
          DefaultColor
          for(int j = 0; j < inst->get_number_of_customers() + 1; j++){
            int number_of_used_edges = 0;
            for(int vehicle = 0; vehicle < inst->get_number_of_vehicles();vehicle++) number_of_used_edges += floor(x[inst->get_edge_id(i, j)][vehicle][period].get(GRB_DoubleAttr_X) + 0.005);
            if(j>i) for(int vehicle = 0; vehicle < inst->get_number_of_vehicles();vehicle++) total_cost += floor(x[inst->get_edge_id(i, j)][vehicle][period].get(GRB_DoubleAttr_X) + 0.005) * inst->get_distance(i,j);
            cout << setw(5) << number_of_used_edges;
          }
          cout << endl;
        }
      }

      cout << "Total cost: " << total_cost << endl;
      ////////////////////////////////////////////////////////////////////////////////
      string pdf_file("");
      pdf_file.append(inst->get_output_path());
      pdf_file.append(inst->get_file_base_name());
      pdf_file.append(".tex");
      if(verbose >= 3) printPackings(inst);
      if(verbose >= 2) generate_pdf(pdf_file, inst, x, y, q, verbose);

    }
    ////////////////////////////////////////////////////////////////////////////////

  }  catch (GRBException e){
    cout << "Error code = " << e.getErrorCode() << endl;
    cout << e.getMessage() << endl;
  }
  catch (...){
    cout << "Exception during optimization" << endl;
  }
  return 0;
}

void generate_pdf(string file, Irp2dInstance *inst, vector < vector < GRBVar * > > x,
    vector < vector < GRBVar * > > y,
    vector < vector < vector < GRBVar * > > > q, int verbose){
  ofstream tex_solution(file.c_str());
  if (!tex_solution.is_open()){
    cerr << "File not found!";
    exit(-1);
  }
  int number_of_vertices = inst->get_number_of_customers() + 1;
  if(verbose >= 3) cout << "Routes in solution: " << endl;
  tex_solution << "\\documentclass[tikz]{standalone}" << endl;
  tex_solution << "\\usepackage{tikz}" << endl;
  tex_solution << "\\usetikzlibrary{decorations.pathreplacing}" << endl;
  tex_solution << "\\usepackage{array,siunitx}" << endl;
  tex_solution << "\\sisetup{add-decimal-zero,round-precision=2,round-mode = places}" << endl;
  tex_solution << "\\begin{document}" << endl;
  for(int period = 1; period < inst->get_horizon()+1; period++){
    if(verbose >= 3) cout << "  period: " << period << endl;
    tex_solution << "\\begin{tikzpicture}[scale = 0.5]" << endl;
    tex_solution << "\\begin{scope}" << endl;
    tex_solution << "\\tikzset{vertex/.style = {shape=circle,draw,minimum size=1.5em}}" << endl;
    tex_solution << "\\tikzset{edge/.style = {-,> = latex}}" << endl;
    double max_dimention = 1;
    for (int vertex = 0; vertex < number_of_vertices; vertex++) max_dimention = max_dimention > inst->get_x(vertex) ? max_dimention : inst->get_x(vertex);
    for (int vertex = 0; vertex < number_of_vertices; vertex++) max_dimention = max_dimention > inst->get_y(vertex) ? max_dimention : inst->get_y(vertex);
    max_dimention = max_dimention / 100;
    tex_solution << "\\node[vertex] (0) at  (" << inst->get_x(0)/max_dimention << "," << inst->get_y(0)/max_dimention   <<") {};" << endl;

    for (int vertex = 1; vertex < number_of_vertices; vertex++){
      tex_solution << "\\node[vertex,fill=black!70!white] (" << vertex << ") at  (" << inst->get_x(vertex)/max_dimention << "," << inst->get_y(vertex)/max_dimention   <<") {};" << endl;
    }



    for(int vehicle = 0; vehicle < inst->get_number_of_vehicles(); vehicle++){
      if(verbose >= 3) cout << "  vehicle: " << vehicle;
      bool vehicle_used = false;
      int mean_x = 0;
      int mean_y = 0;
      int total_vertices = 0;
      for (int i = 0; i < number_of_vertices; i++){
        for (int j = i+1; j < number_of_vertices; j++){
          if(x[inst->get_edge_id(i, j)][vehicle][period].get(GRB_DoubleAttr_X) > 0.9){
            //tex_solution << "\\draw[edge] (" << i << ") to (" << j << ");" << endl;
            tex_solution << "\\draw (" << i << ") -- (" << j << ") node[midway,fill=white]{" << vehicle << "};" << endl;
            vehicle_used = true;
            mean_x += inst->get_x(j);
            mean_y += inst->get_y(i);
            total_vertices++;
          }
        }
      }
      mean_x = (mean_x/(total_vertices))/max_dimention;
      mean_y = (mean_y/(total_vertices))/max_dimention;
      if(!vehicle_used){
        if(verbose >= 3) cout << " not used" << endl;
      }else{
        if(verbose >= 3) {
          cout << " used" << endl << "  nodes: ";
        }
        //reconstructing routes to get packing
        bool seen[number_of_vertices];
        int i, node, len;
        vector <int> route_nodes;
        int route_id = -1;

        for (int vertex = 0; vertex < number_of_vertices; vertex++)
          seen[vertex] = false;

        //start position in tour
        node = 0;

        //while(start < number_of_vertices){
        //searching for the next unseen node
        for (node = 0; node < number_of_vertices; node++)
          if (!seen[node]) break;
        //all nodes already seen
        if (node == number_of_vertices) break;
        //adding nodes to tour
        for (len = 0; len < number_of_vertices; len++) {
          //add node
          //if(node != 0)
          cout << node << " ";
          route_nodes.push_back(node);
          seen[node] = true;
          //search next
          for (i = 0; i < number_of_vertices; i++) {
            if (x[inst->get_edge_id(i,node)][vehicle][period].get(GRB_DoubleAttr_X) > 0.9 && !seen[i]) {
              //set next
              node = i;
              break;
            }
          }
          if (i == number_of_vertices) {
            if(verbose >= 3) cout << "route end." << endl;
            vector < pair < int, int > > route;
            //if no 'next' node were found, cycle ended
            for (unsigned v = 0; v < route_nodes.size(); v++){
              if(verbose >= 3) cout << route_nodes[v] << " ";
              if(route_nodes[v] == 0){
                route.push_back(pair <int, int > (0, -1));
                continue;
              }
              for (int t = period; t <= inst->get_horizon(); t++){
                GRBVar q_var = q[route_nodes[v]][t][vehicle][period];
                if(q_var.get(GRB_DoubleAttr_X) > 0.9){
                  route.push_back(pair <int, int > (route_nodes[v], t));
                  cout << "(" << route_nodes[v] << ", " << t << ")";
                }
              }
            }
            cerr << endl;
            route_id = getPacking(inst, route);
            if(verbose >= 3) cerr << " this is route number " << route_id;
            if (inst->feasible_routes[route_id] != true) {cout << "Invalid route_nodes found!" << endl; exit(-1);};
            break;
          }
          //proceed to next node
        }
        double scale = (double)10 / inst->get_vehicle_width();
        tex_solution << "\\begin{scope}[shift={(" << mean_x  << "," << mean_y << ")},scale = 0.5]" << endl;
        tex_solution << "\\filldraw[fill=white] (0,0) rectangle (" << (double)inst->get_vehicle_width()*scale
            << "," << (double)inst->get_vehicle_height() *scale<< ");" << endl;
        if(verbose >= 3) cout << " number of items in this route_nodes " << inst->x_routes[route_id].size() << endl;
        for(unsigned item = 0; item < inst->x_routes[route_id].size(); item++){
          tex_solution << "\\filldraw[fill=black!" <<  inst->p_routes[route_id][item] * 10 + 10 << "!white, draw=black] ("
              << (double)inst->x_routes[route_id][item]*scale<< "," <<  (double)inst->y_routes[route_id][item]*scale << ") rectangle ("
              << (double)(inst->x_routes[route_id][item]+inst->w_routes[route_id][item])*scale << ","
              << (double)(inst->y_routes[route_id][item]+inst->h_routes[route_id][item])*scale << ");" << endl;
        }
        tex_solution << "\\end{scope}" << endl;
      }
    }
    tex_solution << "\\end{scope}" << endl;
    tex_solution << "\\node[draw] at (-15,-10) {period = " << period << "};";
    tex_solution << "\\end{tikzpicture}" << endl;
  }
  tex_solution << "\\end{document}" << endl;
  tex_solution.close();
  string syscmd("pdflatex -halt-on-error -output-directory ");
  syscmd.append(inst->get_output_path());
  syscmd.append(" ");
  syscmd.append(file);
  syscmd.append(" > /dev/null");
  if(system(syscmd.c_str()) != 0) cerr << "Something is wrong with pdflatex! Sorry.";

  return;
}

} /* namespace 2DIRP */
